'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkTypes = exports.checkType = exports.checkTransport = exports.checkSnapshot = exports.checkStore = exports.checkPubsub = exports.checkOperation = exports.checkMessage = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _isEmpty = require('is-empty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _actions = require('../constants/actions');

var _actions2 = _interopRequireDefault(_actions);

var _nexusError = require('../utils/nexus-error');

var _nexusError2 = _interopRequireDefault(_nexusError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Check a `message` object.
 *
 * @param {Object} message
 */

function checkMessage(message) {
  if ((typeof message === 'undefined' ? 'undefined' : _typeof(message)) !== 'object') {
    throw new _nexusError2.default('message_invalid', { value: message });
  }

  if (typeof message.action !== 'string' || !_actions2.default.includes(message.action)) {
    throw new _nexusError2.default('message_property_invalid', { message: message, property: 'action' });
  }
}

/**
 * Check an `operation` object.
 *
 * @param {Object} operation
 */

function checkOperation(operation) {
  if ((typeof operation === 'undefined' ? 'undefined' : _typeof(operation)) !== 'object') {
    throw new _nexusError2.default('operation_invalid', { operation: operation });
  }

  if (typeof operation.id !== 'string') {
    throw new _nexusError2.default('operation_property_invalid', { operation: operation, property: 'id' });
  }

  if (typeof operation.version !== 'number') {
    throw new _nexusError2.default('operation_property_invalid', { operation: operation, property: 'version' });
  }

  if (_typeof(operation.data) !== 'object') {
    throw new _nexusError2.default('operation_property_invalid', { operation: operation, property: 'data' });
  }
}

/**
 * Check a `pubsub` adapter.
 *
 * @param {Object} pubsub
 */

function checkPubsub(pubsub) {
  if (pubsub == null || (typeof pubsub === 'undefined' ? 'undefined' : _typeof(pubsub)) !== 'object') {
    throw new Error('The `pubsub` object you passed to the Nexus server is invalid.');
  }

  if (typeof pubsub.on !== 'function') {
    throw new Error('Nexus pubsub adapters must implement the `EventEmitter` interface.');
  }

  if (typeof pubsub.subscribe !== 'function') {
    throw new Error('Nexus pubsub adapters must implement the `subscribe()` method.');
  }

  if (typeof pubsub.unsubscribe !== 'function') {
    throw new Error('Nexus pubsub adapters must implement the `unsubscribe()` method.');
  }

  if (typeof pubsub.publish !== 'function') {
    throw new Error('Nexus pubsub adapters must implement the `publish()` method.');
  }
}

/**
 * Check an `snapshot` object.
 *
 * @param {Object} snapshot
 */

function checkSnapshot(snapshot) {
  if ((typeof snapshot === 'undefined' ? 'undefined' : _typeof(snapshot)) !== 'object') {
    throw new _nexusError2.default('snapshot_invalid', { snapshot: snapshot });
  }

  if (typeof snapshot.version !== 'number') {
    throw new _nexusError2.default('snapshot_version_invalid', { snapshot: snapshot, property: 'version' });
  }

  if (typeof snapshot.type !== 'string') {
    throw new _nexusError2.default('snapshot_type_invalid', { snapshot: snapshot, property: 'type' });
  }
}

/**
 * Check a `store` adapter.
 *
 * @param {Object} store
 */

function checkStore(store) {
  if ((typeof store === 'undefined' ? 'undefined' : _typeof(store)) !== 'object') {
    throw new Error('The `store` object you passed to the Nexus server is invalid.');
  }

  if (typeof store.commit !== 'function') {
    throw new Error('Nexus store adapters must implement the `commit()` method.');
  }

  if (typeof store.findSnapshot !== 'function') {
    throw new Error('Nexus store adapters must implement the `findSnapshot()` method.');
  }

  if (typeof store.listOperations !== 'function') {
    throw new Error('Nexus store adapters must implement the `listOperations()` method.');
  }
}

/**
 * Check a `transport` adapter.
 *
 * @param {Object} transport
 */

function checkTransport(transport) {
  if ((typeof transport === 'undefined' ? 'undefined' : _typeof(transport)) !== 'object') {
    throw new Error('The `transport` object you passed to the Nexus server is invalid.');
  }

  if (typeof transport.write !== 'function') {
    throw new Error('Nexus transport adapters must implement the `write()` method.');
  }

  if (typeof transport.read !== 'function') {
    throw new Error('Nexus transport adapters must implement the `read()` method.');
  }

  if (typeof transport.push !== 'function') {
    throw new Error('Nexus transport adapters must implement the `push()` method.');
  }
}

/**
 * Check a `type` string with `types` object.
 *
 * @param {String} type
 * @param {Array} types
 */

function checkType(type, types) {
  if (typeof type !== 'string') {
    throw new _nexusError2.default('type_invalid', { type: type });
  }

  if (!Array.isArray(types) || !types.find(function (t) {
    return t.name === type;
  })) {
    throw new _nexusError2.default('type_unknown', { type: type });
  }
}

/**
 * Check a `types` object.
 *
 * @param {Object} types
 */

function checkTypes(types) {
  if (!Array.isArray(types) || (0, _isEmpty2.default)(types)) {
    throw new Error('The OT `types` array you passed to Nexus is invalid. It should be an array of OT types.');
  }
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.checkMessage = checkMessage;
exports.checkOperation = checkOperation;
exports.checkPubsub = checkPubsub;
exports.checkStore = checkStore;
exports.checkSnapshot = checkSnapshot;
exports.checkTransport = checkTransport;
exports.checkType = checkType;
exports.checkTypes = checkTypes;